@@ -26,7 +26,7 @@ test_that("st_crs returns NA for sfg objects", {
   expect_true(is.na(st_crs(pt1)))
 })
 
-test_that("st_as_binary can read big endian", {
+test_that("st_as_binary can non-native big endian", {
   gc = st_geometrycollection()
   r = st_as_binary(gc)
   r[1] = if (r[1] == 1) {
@@ -38,3 +38,15 @@ test_that("st_as_binary can read big endian", {
   expect_identical(gc, st_as_sfc(structure(list(r), class = "WKB"), pureR = T)[[1]])
   expect_identical(gc, st_as_sfc(structure(list(r), class = "WKB"), pureR = T, EWKB = TRUE)[[1]])
 })
+
+test_that("st_crs<- gives warnings on changing crs", {
+	x = st_sfc(list(st_point(0:1), st_point(0:1)))
+	y = x
+	expect_silent(st_crs(y) <- 4326)
+	expect_silent(st_crs(y) <- 4326)
+	expect_warning(st_crs(y) <- 3857)
+	x = st_sfc(list(st_point(0:1), st_point(0:1)), crs = 4326)
+	expect_silent(st_sfc(x, crs = "+proj=longlat +datum=WGS84 +no_defs"))
+	# but do when it changes:
+	expect_warning(st_sfc(x, crs = "+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +wktext +no_defs"))
+})
